@@ -33,7 +33,7 @@ from tests.unittests.helpers import CiTestCase, mock
 
 
 def int_to_bytes(i):
-    """convert integer to binary: eg: 1 to \x01"""
+    r"""convert integer to binary: eg: 1 to \x01"""
     hex_value = "{0:x}".format(i)
     hex_value = "0" * (len(hex_value) % 2) + hex_value
     return codecs.decode(hex_value, "hex_codec")
@@ -44,7 +44,7 @@ class TestCreateBoundNetlinkSocket(CiTestCase):
     def test_socket_error_on_create(self, m_socket):
         """create_bound_netlink_socket catches socket creation exception"""
 
-        """NetlinkCreateSocketError is raised when socket creation errors."""
+        # NetlinkCreateSocketError is raised when socket creation errors.
         m_socket.side_effect = socket.error("Fake socket failure")
         with self.assertRaises(NetlinkCreateSocketError) as ctx_mgr:
             create_bound_netlink_socket()
